/*
 * Modification History
 *
 * 2003-October-26   Jason Rohrer
 * Created.
 *
 * 2003-October-27   Jason Rohrer
 * Added support for starting a download.
 *
 * 2003-November-6   Jason Rohrer
 * Added support for new result fields.
 *
 * 2003-November-9   Jason Rohrer
 * Removed excess print statements.
 *
 * 2003-December-16   Jason Rohrer
 * Fixed a scrolling bug in OS X.
 *
 * 2003-December-23   Jason Rohrer
 * Added UI support for hash mismatch.
 *
 * 2004-January-12   Jason Rohrer
 * Fixed a search locking bug.
 */



#include "SearchPanel.h"

#include "MUTE/otherApps/fileSharing/userInterface/common/formatUtils.h"




// for non-precomp compilers, include the necessary headers
#ifndef WX_PRECOMP
//    #include "wx/app.h"
//  #include "wx/log.h"
//    #include "wx/frame.h"
    #include "wx/panel.h"
//    #include "wx/stattext.h"
//    #include "wx/menu.h"
//  #include "wx/layout.h"
//    #include "wx/dialog.h"
//    #include "wx/msgdlg.h"
    #include "wx/sizer.h"
//    #include "wx/textdlg.h"
//  #include "wx/progdlg.h"
//    #include "wx/utils.h"
//    #include "wx/choicdlg.h"
    #include "wx/button.h"
#endif


// IDs for various controls
enum {
    BUTTON_SEARCH = 1,
    BUTTON_DOWNLOAD,
    FIELD_SEARCH
    };



// event mapping table
BEGIN_EVENT_TABLE( SearchPanel, wxPanel )
    EVT_BUTTON( BUTTON_SEARCH,  SearchPanel::OnSearch )
    EVT_BUTTON( BUTTON_DOWNLOAD,  SearchPanel::OnDownload )
    EVT_TEXT_ENTER( FIELD_SEARCH, SearchPanel::OnSearch )
END_EVENT_TABLE();



SearchPanel::SearchPanel( wxNotebook *inNotebook,
                          DownloadPanel *inDownloadPanel )
    : wxPanel( inNotebook, -1 ),
      mSearchLock( new MutexLock() ),
      mSearchActive( false ),
      mSearchCanceled( false ),
      mResultLock( new MutexLock() ),
      mResultsFromAddresses( new SimpleVector<char *>() ),
      mResultsFilePaths( new SimpleVector<char *>() ),
      mResultsFileHashes( new SimpleVector<char *>() ),
      mSearchThread( NULL ),
      mSearchField( new wxTextCtrl( this, FIELD_SEARCH,
                                    "", wxDefaultPosition,
                                    wxDefaultSize,
                                    wxTE_PROCESS_ENTER ) ),
      mSearchResultsList( new wxListCtrl( this, -1,
                                          wxDefaultPosition,
                                          wxDefaultSize,
                                          wxLC_REPORT | wxLC_VRULES ) ),
      mDownloadPanel( inDownloadPanel ) {

    
    wxBoxSizer *panelSizer = new wxBoxSizer( wxVERTICAL );

    this->SetSizer( panelSizer );


    wxBoxSizer *searchFieldSizer = new wxBoxSizer( wxHORIZONTAL );

    panelSizer->Add( searchFieldSizer,
                     0,
                     wxEXPAND | wxALL,
                     10 );

    searchFieldSizer->Add( mSearchField,
                           1,
                           wxEXPAND,
                           2 );

    wxButton *searchButton = new wxButton( this, BUTTON_SEARCH, "Search" );

    searchFieldSizer->Add( searchButton,
                           0,
                           wxALIGN_RIGHT,
                           2 );


    mSearchResultsList->InsertColumn( 0, "File Name" );
    mSearchResultsList->InsertColumn( 1, "Size" );
    mSearchResultsList->InsertColumn( 2, "Hash" );
    mSearchResultsList->InsertColumn( 3, "Host Virtual Address" );
    mSearchResultsList->SetColumnWidth( 0, 200 );
    mSearchResultsList->SetColumnWidth( 1, 100 );
    mSearchResultsList->SetColumnWidth( 2, 200 );
    mSearchResultsList->SetColumnWidth( 3, 200 );
    
    
    panelSizer->Add( mSearchResultsList,
                     1,            // make vertically stretchable
                     wxEXPAND,     // make horizontally stretchable
                     2 );          // set border width to 2


    wxBoxSizer *downloadButtonSizer = new wxBoxSizer( wxHORIZONTAL );

    panelSizer->Add( downloadButtonSizer,
                     0,
                     wxEXPAND | wxALL,
                     10 );

    wxButton *downloadButton =
        new wxButton( this, BUTTON_DOWNLOAD, "Download" );


    // spacer panel
    downloadButtonSizer->Add( new wxPanel( this, -1 ),
                              1,
                              wxEXPAND,
                              2 );
    
    downloadButtonSizer->Add( downloadButton,
                           0,
                           wxALIGN_RIGHT,
                           2 );
    
    }



SearchPanel::~SearchPanel() {
    // cancel any active search
    cancelActiveSearch();
    
    delete mSearchLock;


    int numResults = mResultsFromAddresses->size();
    for( int i=0; i<numResults; i++ ) {
        delete [] *( mResultsFromAddresses->getElement( i ) );
        delete [] *( mResultsFilePaths->getElement( i ) );
        delete [] *( mResultsFileHashes->getElement( i ) );
        }
    delete mResultsFromAddresses;
    delete mResultsFilePaths;
    delete mResultsFileHashes;    

    delete mResultLock;
    }



void SearchPanel::cancelActiveSearch() {
    mSearchLock->lock();
    if( mSearchActive ) {
        mSearchCanceled = true;
        mSearchLock->unlock();

        // wait for search thread to return
        delete mSearchThread;
        }
    else {
        mSearchLock->unlock();
        }
    }



// callback for incoming search results
char searchResultHandler( char *inFromAddress, char *inFilePath,
                          unsigned long inFileSize,
                          char *inFileHash,
                          void *inExtraParam ) {
    // unwrap panel from extra parameter
    SearchPanel *parentPanel = (SearchPanel *)inExtraParam;
    
    if( inFromAddress != NULL ) {
        parentPanel->mResultLock->lock();

        int nextItem = parentPanel->mSearchResultsList->GetItemCount();
        
        parentPanel->mSearchResultsList->InsertItem( nextItem, inFilePath );

        char *sizeString = formatDataSizeWithUnits( inFileSize );
        
        parentPanel->mSearchResultsList->SetItem( nextItem, 1, sizeString );
        parentPanel->mSearchResultsList->SetItem( nextItem, 2, inFileHash );
        parentPanel->mSearchResultsList->SetItem( nextItem, 3, inFromAddress );

        delete [] sizeString;
        
        parentPanel->mResultsFromAddresses->push_back(
            stringDuplicate( inFromAddress ) );
        parentPanel->mResultsFilePaths->push_back(
            stringDuplicate( inFilePath ) );
        parentPanel->mResultsFileHashes->push_back(
            stringDuplicate( inFileHash ) );
        
        parentPanel->mResultLock->unlock();
        }

    char returnValue = true;
        
    parentPanel->mSearchLock->lock();

    if( parentPanel->mSearchCanceled ) {
        returnValue = false;
        }
    parentPanel->mSearchLock->unlock();

    return returnValue;
    }




void SearchPanel::OnSearch( wxCommandEvent& inEvent ) {

    char *searchString = stringDuplicate( mSearchField->GetValue().c_str() );



    mSearchLock->lock();
    if( mSearchActive ) {
        // cancel the search
        mSearchCanceled = true;
        mSearchLock->unlock();
        
        // wait for the thread to return
        delete mSearchThread;
        }
    else {
        mSearchLock->unlock();
        }
    mSearchCanceled = false;
                    
    // delete old results, if there are any
    mResultLock->lock();

    // scroll back to item 0 before deleting items to fix a bug
    // in OS X behavior
    if( mSearchResultsList->GetItemCount() > 0 ) {
        mSearchResultsList->EnsureVisible( 0 );
        }
    
    mSearchResultsList->DeleteAllItems();

    int numResults = mResultsFromAddresses->size();
    for( int i=0; i<numResults; i++ ) {
        delete [] *( mResultsFromAddresses->getElement( i ) );
        delete [] *( mResultsFilePaths->getElement( i ) );
        delete [] *( mResultsFileHashes->getElement( i ) );
        }
    mResultsFromAddresses->deleteAll();
    mResultsFilePaths->deleteAll();
    mResultsFileHashes->deleteAll();
    
    mResultLock->unlock();
                
                
    // spawn thread for search
    mSearchThread = new SearchThread( searchString,
                                      mSearchLock,
                                      &mSearchActive,
                                      (void *)this );

    delete [] searchString;
    }



void SearchPanel::OnDownload( wxCommandEvent& inEvent ) {

    mResultLock->lock();

    int count = mSearchResultsList->GetItemCount();

    for( int i=0; i<count; i++ ) {
        if( mSearchResultsList->GetItemState( i, wxLIST_STATE_SELECTED ) ) {

            char *fromAddress = *( mResultsFromAddresses->getElement( i ) );
            char *filePath = *( mResultsFilePaths->getElement( i ) );
            char *fileHash = *( mResultsFileHashes->getElement( i ) );

            
            mDownloadPanel->addDownload( fromAddress, filePath, fileHash );
            }
        }

    
    
    mResultLock->unlock();
    
    }


