/*
 * Modification History
 *
 * 2003-November-4   Jason Rohrer
 * Created.
 *
 * 2003-November-5   Jason Rohrer
 * Fixed destruction/stopping behavior of update thread.
 */



#include "UploadsPanel.h"





// for non-precomp compilers, include the necessary headers
#ifndef WX_PRECOMP
    #include "wx/app.h"
//  #include "wx/log.h"
//    #include "wx/frame.h"
//    #include "wx/panel.h"
//    #include "wx/stattext.h"
//    #include "wx/menu.h"
//  #include "wx/layout.h"
//    #include "wx/dialog.h"
//    #include "wx/msgdlg.h"
    #include "wx/sizer.h"
//    #include "wx/textdlg.h"
//  #include "wx/progdlg.h"
//    #include "wx/utils.h"
//    #include "wx/choicdlg.h"
//    #include "wx/button.h"
#endif



#include "MUTE/otherApps/fileSharing/fileShare.h"



#include "minorGems/util/stringUtils.h"



// IDs for various controls
enum {
    UPLOADS_UPDATE_EVENT = 1
    };



// event mapping table
BEGIN_EVENT_TABLE( UploadsPanel, wxPanel )
    // dummy events sent by thread to trigger an update
    EVT_MENU( UPLOADS_UPDATE_EVENT, UploadsPanel::OnUploadsUpdate )
END_EVENT_TABLE();



UploadsPanel::UploadsPanel( wxNotebook *inNotebook )
    : wxPanel( inNotebook, -1 ),
      mUploadList( new wxListCtrl( this, -1,
                                       wxDefaultPosition,
                                       wxDefaultSize,
                                       wxLC_REPORT | wxLC_VRULES ) ),
      mUpdateThread( NULL ) {

    
    wxBoxSizer *panelSizer = new wxBoxSizer( wxVERTICAL );

    this->SetSizer( panelSizer );




    mUploadList->InsertColumn( 0, "File" );
    mUploadList->InsertColumn( 1, "Chunks Uploaded" );
    mUploadList->SetColumnWidth( 0, 150 );
    mUploadList->SetColumnWidth( 1, 150 );
    
    panelSizer->Add( mUploadList,
                     1,            // make vertically stretchable
                     wxEXPAND,     // make horizontally stretchable
                     0 );

    // start our thread
    mUpdateThread = new UploadUpdateThread( (void *)this );
    }



UploadsPanel::~UploadsPanel() {
    delete mUpdateThread;
    }






void UploadsPanel::OnUploadsUpdate( wxCommandEvent& inEvent ) {

    mUploadList->DeleteAllItems();


    char **filePaths;
    int *chunkCounts;

    int numUploads = muteShareGetUploadChunkCounts( &filePaths, &chunkCounts );

    for( int i=0; i<numUploads; i++ ) {

        char *countString = autoSprintf( "%d", chunkCounts[i] );
        
        mUploadList->InsertItem( i, filePaths[i] );
        mUploadList->SetItem( i, 1, countString );

        delete [] filePaths[i];
        delete [] countString;
        }

    delete [] filePaths;
    delete [] chunkCounts;
    }



UploadUpdateThread::UploadUpdateThread( void *inParentPanel )
    : mParentPanel( inParentPanel ),
      mStopLock( new MutexLock() ),
      mStopped( false ) {

    start();
    }



UploadUpdateThread::~UploadUpdateThread() {
    mStopLock->lock();
    mStopped = true;
    mStopLock->unlock();

    // yeild so that any events generated by thread can be posted
    // else thread will block on event post and never return
    wxYield();
    
    join();

    delete mStopLock;
    }



void UploadUpdateThread::run() {
    mStopLock->lock();
    char isStopped = mStopped;
    mStopLock->unlock();

    UploadsPanel *panel = (UploadsPanel *)mParentPanel;
    
    while( ! isStopped ) {

        // fire an update event

        // post event so that GUI thread can update UI
        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED,
                              UPLOADS_UPDATE_EVENT );
        
        // send in a thread-safe way
        wxPostEvent( panel, event );

        // sleep for 5 seconds in 1-second increments
        for( int i=0; i<5 && !isStopped; i++ ) {
            sleep( 1000 );


            // check if stopped
            mStopLock->lock();
            isStopped = mStopped;
            mStopLock->unlock();
            }
        }
    }





